@@ -508,7 +508,7 @@ class AutogenerateDiffTest(ModelOne, AutogenTest, TestBase):
         autogenerate.compare._compare_tables(
             OrderedSet([(None, 'extra'), (None, 'user')]),
             OrderedSet(), inspector,
-            MetaData(), uo, self.autogen_context
+            uo, self.autogen_context
         )
         eq_(
             [(rec[0], rec[1].name) for rec in uo.as_diffs()],
@@ -754,9 +754,10 @@ class PKConstraintUpgradesIgnoresNullableTest(AutogenTest, TestBase):
         return cls._get_db_schema()
 
     def test_no_change(self):
-        diffs = []
+        uo = ops.UpgradeOps(ops=[])
         ctx = self.autogen_context
-        autogenerate._produce_net_changes(ctx, diffs)
+        autogenerate._produce_net_changes(ctx, uo)
+        diffs = uo.as_diffs()
         eq_(diffs, [])
 
 
@@ -822,21 +823,21 @@ class AutogenVersionTableTest(AutogenTest, TestBase):
         return m
 
     def test_no_version_table(self):
-        diffs = []
         ctx = self.autogen_context
 
-        autogenerate._produce_net_changes(ctx, diffs)
-        eq_(diffs, [])
+        uo = ops.UpgradeOps(ops=[])
+        autogenerate._produce_net_changes(ctx, uo)
+        eq_(uo.as_diffs(), [])
 
     def test_version_table_in_target(self):
-        diffs = []
         Table(
             self.version_table_name,
             self.m2, Column('x', Integer), schema=self.version_table_schema)
 
         ctx = self.autogen_context
-        autogenerate._produce_net_changes(ctx, diffs)
-        eq_(diffs, [])
+        uo = ops.UpgradeOps(ops=[])
+        autogenerate._produce_net_changes(ctx, uo)
+        eq_(uo.as_diffs(), [])
 
 
 class AutogenCustomVersionTableSchemaTest(AutogenVersionTableTest):
@@ -1203,3 +1204,258 @@ class OrigObjectTest(TestBase):
         is_(op.to_index(), self.ix)
         is_(op.reverse().to_index(), self.ix)
 
+
+class MultipleMetaDataTest(AutogenFixtureTest, TestBase):
+    def test_multiple(self):
+        m1a = MetaData()
+        m1b = MetaData()
+        m1c = MetaData()
+
+        m2a = MetaData()
+        m2b = MetaData()
+        m2c = MetaData()
+
+        Table('a', m1a, Column('id', Integer, primary_key=True))
+        Table('b1', m1b, Column('id', Integer, primary_key=True))
+        Table('b2', m1b, Column('id', Integer, primary_key=True))
+        Table('c1', m1c, Column('id', Integer, primary_key=True),
+              Column('x', Integer))
+
+        a = Table('a', m2a, Column('id', Integer, primary_key=True),
+                  Column('q', Integer))
+        Table('b1', m2b, Column('id', Integer, primary_key=True))
+        Table('c1', m2c, Column('id', Integer, primary_key=True))
+        c2 = Table('c2', m2c, Column('id', Integer, primary_key=True))
+
+        diffs = self._fixture([m1a, m1b, m1c], [m2a, m2b, m2c])
+        eq_(diffs[0], ('add_table', c2))
+        eq_(diffs[1][0], 'remove_table')
+        eq_(diffs[1][1].name, 'b2')
+        eq_(diffs[2], ('add_column', None, 'a', a.c.q))
+        eq_(diffs[3][0:3], ('remove_column', None, 'c1'))
+        eq_(diffs[3][3].name, 'x')
+
+    def test_empty_list(self):
+        # because they're going to do it....
+
+        diffs = self._fixture([], [])
+        eq_(diffs, [])
+
+    def test_non_list_sequence(self):
+        # we call it "sequence", let's check that
+
+        m1a = MetaData()
+        m1b = MetaData()
+
+        m2a = MetaData()
+        m2b = MetaData()
+
+        Table('a', m1a, Column('id', Integer, primary_key=True))
+        Table('b', m1b, Column('id', Integer, primary_key=True))
+
+        Table('a', m2a, Column('id', Integer, primary_key=True))
+        b = Table('b', m2b, Column('id', Integer, primary_key=True),
+                  Column('q', Integer))
+
+        diffs = self._fixture((m1a, m1b), (m2a, m2b))
+        eq_(
+            diffs,
+            [('add_column', None, 'b', b.c.q)]
+        )
+
+    def test_raise_on_dupe(self):
+        m1a = MetaData()
+        m1b = MetaData()
+
+        m2a = MetaData()
+        m2b = MetaData()
+
+        Table('a', m1a, Column('id', Integer, primary_key=True))
+        Table('b1', m1b, Column('id', Integer, primary_key=True))
+        Table('b2', m1b, Column('id', Integer, primary_key=True))
+        Table('b3', m1b, Column('id', Integer, primary_key=True))
+
+        Table('a', m2a, Column('id', Integer, primary_key=True))
+        Table('a', m2b, Column('id', Integer, primary_key=True))
+        Table('b1', m2b, Column('id', Integer, primary_key=True))
+        Table('b2', m2a, Column('id', Integer, primary_key=True))
+        Table('b2', m2b, Column('id', Integer, primary_key=True))
+
+        assert_raises_message(
+            ValueError,
+            'Duplicate table keys across multiple MetaData objects: "a", "b2"',
+            self._fixture,
+            [m1a, m1b], [m2a, m2b]
+        )
+
+
+class AutoincrementTest(AutogenFixtureTest, TestBase):
+    __backend__ = True
+    __requires__ = 'integer_subtype_comparisons',
+
+    def test_alter_column_autoincrement_none(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table('a', m1, Column('x', Integer, nullable=False))
+        Table('a', m2, Column('x', Integer, nullable=True))
+
+        ops = self._fixture(m1, m2, return_ops=True)
+        assert 'autoincrement' not in ops.ops[0].ops[0].kw
+
+    def test_alter_column_autoincrement_pk_false(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table(
+            'a', m1,
+            Column('x', Integer, primary_key=True, autoincrement=False))
+        Table(
+            'a', m2,
+            Column('x', BigInteger, primary_key=True, autoincrement=False))
+
+        ops = self._fixture(m1, m2, return_ops=True)
+        is_(ops.ops[0].ops[0].kw['autoincrement'], False)
+
+    def test_alter_column_autoincrement_pk_implicit_true(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table(
+            'a', m1,
+            Column('x', Integer, primary_key=True))
+        Table(
+            'a', m2,
+            Column('x', BigInteger, primary_key=True))
+
+        ops = self._fixture(m1, m2, return_ops=True)
+        is_(ops.ops[0].ops[0].kw['autoincrement'], True)
+
+    def test_alter_column_autoincrement_pk_explicit_true(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table(
+            'a', m1,
+            Column('x', Integer, primary_key=True, autoincrement=True))
+        Table(
+            'a', m2,
+            Column('x', BigInteger, primary_key=True, autoincrement=True))
+
+        ops = self._fixture(m1, m2, return_ops=True)
+        is_(ops.ops[0].ops[0].kw['autoincrement'], True)
+
+    def test_alter_column_autoincrement_nonpk_false(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table(
+            'a', m1,
+            Column('id', Integer, primary_key=True),
+            Column('x', Integer, autoincrement=False)
+        )
+        Table(
+            'a', m2,
+            Column('id', Integer, primary_key=True),
+            Column('x', BigInteger, autoincrement=False)
+        )
+
+        ops = self._fixture(m1, m2, return_ops=True)
+        is_(ops.ops[0].ops[0].kw['autoincrement'], False)
+
+    def test_alter_column_autoincrement_nonpk_implicit_false(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table(
+            'a', m1,
+            Column('id', Integer, primary_key=True),
+            Column('x', Integer)
+        )
+        Table(
+            'a', m2,
+            Column('id', Integer, primary_key=True),
+            Column('x', BigInteger)
+        )
+
+        ops = self._fixture(m1, m2, return_ops=True)
+        assert 'autoincrement' not in ops.ops[0].ops[0].kw
+
+    @config.requirements.fail_before_sqla_110
+    def test_alter_column_autoincrement_nonpk_explicit_true(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table(
+            'a', m1,
+            Column('id', Integer, primary_key=True),
+            Column('x', Integer, autoincrement=True)
+        )
+        Table(
+            'a', m2,
+            Column('id', Integer, primary_key=True),
+            Column('x', BigInteger, autoincrement=True)
+        )
+
+        ops = self._fixture(m1, m2, return_ops=True)
+        is_(ops.ops[0].ops[0].kw['autoincrement'], True)
+
+    def test_alter_column_autoincrement_compositepk_false(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table(
+            'a', m1,
+            Column('id', Integer, primary_key=True),
+            Column('x', Integer, primary_key=True, autoincrement=False)
+        )
+        Table(
+            'a', m2,
+            Column('id', Integer, primary_key=True),
+            Column('x', BigInteger, primary_key=True, autoincrement=False)
+        )
+
+        ops = self._fixture(m1, m2, return_ops=True)
+        is_(ops.ops[0].ops[0].kw['autoincrement'], False)
+
+    def test_alter_column_autoincrement_compositepk_implicit_false(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table(
+            'a', m1,
+            Column('id', Integer, primary_key=True),
+            Column('x', Integer, primary_key=True)
+        )
+        Table(
+            'a', m2,
+            Column('id', Integer, primary_key=True),
+            Column('x', BigInteger, primary_key=True)
+        )
+
+        ops = self._fixture(m1, m2, return_ops=True)
+        assert 'autoincrement' not in ops.ops[0].ops[0].kw
+
+    @config.requirements.autoincrement_on_composite_pk
+    def test_alter_column_autoincrement_compositepk_explicit_true(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table(
+            'a', m1,
+            Column('id', Integer, primary_key=True, autoincrement=False),
+            Column('x', Integer, primary_key=True, autoincrement=True),
+            # on SQLA 1.0 and earlier, this being present
+            # trips the "add KEY for the primary key" so that the
+            # AUTO_INCREMENT keyword is accepted by MySQL.  SQLA 1.1 and
+            # greater the columns are just reorganized.
+            mysql_engine='InnoDB'
+        )
+        Table(
+            'a', m2,
+            Column('id', Integer, primary_key=True, autoincrement=False),
+            Column('x', BigInteger, primary_key=True, autoincrement=True)
+        )
+
+        ops = self._fixture(m1, m2, return_ops=True)
+        is_(ops.ops[0].ops[0].kw['autoincrement'], True)
