@@ -222,8 +222,6 @@ class RevisionCommandTest(TestBase):
         )
 
 
-
-
 class CustomizeRevisionTest(TestBase):
     def setUp(self):
         self.env = staging_env()
@@ -412,6 +410,45 @@ def downgrade():
                 ["alembic_version"]
             )
 
+    def test_programmatic_command_option(self):
+
+        def process_revision_directives(context, rev, generate_revisions):
+            generate_revisions[0].message = "test programatic"
+            generate_revisions[0].upgrade_ops = ops.UpgradeOps(
+                ops=[
+                    ops.CreateTableOp(
+                        'test_table',
+                        [
+                            sa.Column('id', sa.Integer(), primary_key=True),
+                            sa.Column('name', sa.String(50), nullable=False)
+                        ]
+                    ),
+                ]
+            )
+            generate_revisions[0].downgrade_ops = ops.DowngradeOps(
+                ops=[
+                    ops.DropTableOp('test_table')
+                ]
+            )
+
+        with self._env_fixture(None, None):
+            rev = command.revision(
+                self.cfg,
+                head="model1@head",
+                process_revision_directives=process_revision_directives)
+
+        result = open(rev.path).read()
+        assert ("""
+def upgrade():
+    # ### commands auto generated by Alembic - please adjust! ###
+    op.create_table('test_table',
+    sa.Column('id', sa.Integer(), nullable=False),
+    sa.Column('name', sa.String(length=50), nullable=False),
+    sa.PrimaryKeyConstraint('id')
+    )
+    # ### end Alembic commands ###
+""") in result
+
 
 class ScriptAccessorTest(TestBase):
     def test_upgrade_downgrade_ops_list_accessors(self):
@@ -900,4 +937,4 @@ def downgrade():
             eq_(
                 [rev.revision for rev in script.walk_revisions()],
                 [self.model1, self.model2, self.model3]
-            )
\ No newline at end of file
+            )
